﻿// Copyright (C) 2003, 2005 Daisuke Arai <darai@users.sourceforge.jp>
// Copyright (C) 2008, 2013 panacoran <panacoran@users.sourceforge.jp>
// 
// This program is part of Protra.
//
// Protra is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
// 
// $Id: EquivalenceComparisonNode.cs 455 2013-06-08 11:52:33Z panacoran $

using System.Collections.Generic;

namespace Protra.Lib.Lang
{
    /// <summary>
    /// 等値比較のノードを表すクラス。
    /// </summary>
    public class EquivalenceComparisonNode : Node
    {
        /// <summary>
        /// ノードのリスト
        /// </summary>
        private List<Node> _nodeList;

        /// <summary>
        /// 演算の種類を表すキャラクタのリスト
        /// </summary>
        private List<char> _opCharList;

        /// <summary>
        /// 構文解析する。
        /// </summary>
        /// <exception cref="Protra.Lib.Lang.ParseException">
        /// 構文解析中にエラーが発生したときにthrowされる。
        /// </exception>
        /// <return>
        /// 解析した結果のノード。
        /// </return>
        public override Node Parse()
        {
            Token = Scanner.Token;
            var node = new ComparisonNode().Parse();
            var token = Scanner.Token.Value;
            if (!(token == "==" || token == "!="))
                return node;
            _nodeList = new List<Node>();
            _opCharList = new List<char>();
            _nodeList.Add(node);
            do
            {
                _opCharList.Add(token[0]);
                Scanner.Scan();
                _nodeList.Add(new ComparisonNode().Parse());
                token = Scanner.Token.Value;
            } while (token == "==" || token == "!=");
            return this;
        }

        /// <summary>
        /// プログラムを実行する。
        /// </summary>
        /// <exception cref="Protra.Lib.Lang.RuntimeException">
        /// プログラム実行中にエラーが発生した場合にthrowされる。
        /// </exception>
        /// <param name="resource">リソース</param>
        /// <param name="at">int型@作用素の値</param>
        /// <param name="ats">string型@作用素の値</param>
        /// <returns>実行結果</returns>
        public override Value Execute(Resource resource, int at, string ats)
        {
            try
            {
                var val1 = _nodeList[0].Execute(resource, at, ats);
                for (var i = 1; i < _nodeList.Count; i++)
                {
                    var val2 = _nodeList[i].Execute(resource, at, ats);
                    val1 = _opCharList[i - 1] == '=' ? new Value(val1 == val2) : new Value(val1 != val2);
                }
                return val1;
            }
            catch (RuntimeException e)
            {
                throw new RuntimeException(e.Message, e.Token ?? Token, e);
            }
        }
    }
}